#!/usr/bin/env python3

import objc, re, os
from AppKit import *
from Quartz.CoreGraphics import *
from comipoli_view import ComipoliView
from comipoli_archive import ComipoliArchive
from comipoli_toolbar import ComipoliToolbar
from comipoli_collection import ComipoliCollection

class ComipoliWindow(NSWindow):

    # Avoid garbage collection
    avoid_gc = []

    def init(self):
        # Create
        objc.super(ComipoliWindow, self).initWithContentRect_styleMask_backing_defer_(
            NSApp.init_frame,
            NSTitledWindowMask |
            NSClosableWindowMask |
            NSResizableWindowMask |
            NSMiniaturizableWindowMask,
            NSBackingStoreBuffered,
            False)
        # Cascade Position
        if len(NSApp.windows()) > 1:
            point = NSApp.keyWindow().cascadeTopLeftFromPoint_(NSZeroPoint)
            self.setFrameTopLeftPoint_(point)
        self.setMinSize_(NSMakeSize(600, 300))
        # var
        self.num = 0
        self.is_fullscreen = False
        self.next_filename = ''
        self.thumbnail_array = []
        self.thumbnail_num = 0
        # Event
        self.setDelegate_(self)
        # archive
        self.archive = ComipoliArchive()
        # NSView
        self.canvas = ComipoliView.new()
        self.canvas.setFrameSize_(NSApp.init_frame.size)
        self.contentView().addSubview_(self.canvas)
        # DnD
        self.registerForDraggedTypes_([NSPasteboardTypeURL])
        # Toolbar
        self.toolbar = ComipoliToolbar.new()
        self.toolbar.open_button.setTarget_(self)
        self.setTitleVisibility_(1)
        self.setToolbar_(self.toolbar)
        defaults = NSUserDefaults.alloc().initWithSuiteName_('com.sasakima.comipoli')
        if defaults.boolForKey_('LtoR'):
            self.onLRButtonClicked_(None)
        return self

    @classmethod
    def newWindow(self):
        window = ComipoliWindow.new()
        self.avoid_gc.append(window)
        window.makeKeyAndOrderFront_(window)
        return window

    @objc.python_method
    def set_path(self, path):
        if self.archive.new_archive(path):
            self.num = 0
            self.canvas.is_pdf = self.archive.status == 3
            self.change_pixbuf(0)
            # thunbnail
            self.thumbnail_array = []
            self.thumbnail_num = 0
            self.toolbar.thumb_button.setEnabled_(False)
            self.performSelectorInBackground_withObject_(self.readThumbnailImage_, path)
        else:
            self.messagebox('i can not Open\n' + path)

    def readThumbnailImage_(self, path):
        l = len(self.archive)
        for num in range(l):
            image = self.archive.get_thumb_image(num, NSApp.thumbnail_height, NSApp.thumbnail_height)
            self.performSelectorOnMainThread_withObject_waitUntilDone_(self.addthumbnailImage_, image, True)
        self.performSelectorOnMainThread_withObject_waitUntilDone_(self.endthumbnailImage_, None, True)

    def addthumbnailImage_(self, image):
        self.thumbnail_array.append({'pic': image, 'num': self.thumbnail_num})
        self.thumbnail_num += 1

    def endthumbnailImage_(self, dumme):
        self.toolbar.thumb_button.setEnabled_(True)

    @objc.python_method
    def toggle_full_screen(self):
        self.toggleFullScreen_(self)

    @objc.python_method
    def messagebox(self, text):
        alert = NSAlert.new()
        alert.setMessageText_(text)
        alert.addButtonWithTitle_('OK')
        alert.setInformativeText_('from comipoli')
        alert.beginSheetModalForWindow_completionHandler_(self, None)

    @objc.python_method
    def is_oblong(self, buf):
        if buf == None:
            return True
        if self.archive.status == 3:
            rect = CGPDFPageGetBoxRect(buf, kCGPDFMediaBox)
            return rect.size.width - rect.size.height > 0
        return buf.size().width - buf.size().height > 0

    @objc.python_method
    def get_file_numsort_list(self, dirname, exts, etc):
        files = []
        re_ext = '\.({0})$'.format(exts)
        l = os.listdir(dirname)
        for s in l:
            if re.search(re_ext, s, re.I):
                files.append(s)
            elif s == etc:
                files.append(s)
        convert = lambda text: int(text) if text.isdigit() else text.lower() 
        alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', key) ] 
        return sorted(files, key = alphanum_key)

    @objc.python_method
    def change_pixbuf(self, nex):
        length = len(self.archive)
        if length == 0:
            return
        try:
            num = self.num + nex if self.canvas.spread else self.num + nex - 1
            if length < num + 1 and nex != 50:
                path = os.path.dirname(self.archive.path)
                etc = os.path.basename(self.archive.path) # if (zip|rar|7z) ext
                exts = 'cbz'
                if NSApp.is_pdf:
                    exts += '|pdf'
                if NSApp.path_rar != '':
                    exts += '|cbr'
                if NSApp.path_7za != '':
                    exts += '|cb7'
                cbzs = self.get_file_numsort_list(path, exts, etc)
                l = len(cbzs)
                if l == 0:
                    return
                else:
                    next_index = cbzs.index(os.path.basename(self.archive.path)) + 1
                    if l > next_index:
                        self.next_filename = '{0}/{1}'.format(path, cbzs[next_index])
                        alert = NSAlert.new()
                        alert.setMessageText_('next: ' + cbzs[next_index])
                        alert.setInformativeText_('Next File')
                        alert.addButtonWithTitle_('OK')
                        alert.addButtonWithTitle_('Cancel')
                        alert.beginSheetModalForWindow_completionHandler_(self, self.nextCb_)
                    elif self.is_fullscreen:
                        self.toggle_full_screen()
            else:
                if nex == 0:
                    # Start Page
                    self.canvas.first_page = self.archive[0]
                    self.num = 0
                    self.canvas.spread = False
                    self.canvas.display()
                elif nex == 50:
                    # Last Page
                    self.canvas.first_page = self.archive[length - 1]
                    self.num = length - 1
                    self.canvas.spread = False
                    self.canvas.display()
                elif nex > 0:
                    # Next Page
                    if length > num:
                        # Set Right Page
                        self.canvas.first_page = self.archive[num]
                        if self.is_oblong(self.canvas.first_page):
                            self.canvas.spread = False
                        else:
                            if length > num + 1:
                                # Spread View
                                self.canvas.second_page = self.archive[num + 1]
                                if self.is_oblong(self.canvas.second_page):
                                    self.canvas.spread = False
                                else:
                                    self.canvas.spread = True
                            else:
                                self.canvas.spread = False
                        self.num = num
                        self.canvas.display()
                else:
                    # Previos Page
                    num2 = self.num + nex
                    if num2 < 0:
                        self.num = 0
                        self.canvas.first_page = self.archive[0]
                        self.canvas.spread = False
                        self.canvas.display()
                    else:
                        # Check (width > height)
                        pixbuf = self.archive[num2]
                        pixbuf2 = self.archive[num2 + 1] if length > 1 else None
                        if self.is_oblong(pixbuf) or self.is_oblong(pixbuf2):
                            # Single View
                            self.canvas.first_page = pixbuf2
                            self.canvas.spread = False
                            self.num = num2 + 1
                        else:
                            # Spread View
                            self.canvas.first_page = pixbuf
                            self.canvas.second_page = pixbuf2
                            self.canvas.spread = True
                            self.num = num2
                        self.canvas.display()
                # Draw Page Number
                title = os.path.basename(self.archive.path)
                self.toolbar.title_label.setStringValue_('{0} {1}/{2}'.format(title, self.num + 1, length))
        except Exception as e:
            self.messaagebox(e)

    def nextCb_(self, res):
        if res == NSAlertFirstButtonReturn:
            self.set_path(self.next_filename)

    def openCb_(self, res):
        if res == NSFileHandlingPanelOKButton:
            url = self.open_panel.URLs()[0]
            self.set_path(url.path())

    def onOpenButtonClicked_(self, button):
        menu = NSApp.mainMenu().itemAtIndex_(2)
        menu.setEnabled_(False)
        self.open_panel = NSOpenPanel.openPanel()
        self.open_panel.setCanChooseDirectories_(True)
        self.open_panel.setAllowsMultipleSelection_(False)
        arr = ['cbz','zip']
        if NSApp.path_rar != '':
            arr.append('cbr')
            arr.append('rar')
        if NSApp.path_7za != '':
            arr.append('cb7')
            arr.append('7z')
        if NSApp.is_pdf:
            arr.append('pdf')
        self.open_panel.setAllowedFileTypes_(arr)
        self.open_panel.beginSheetModalForWindow_completionHandler_(self, self.openCb_)
        menu.setEnabled_(True)

    def onLRButtonClicked_(self, button):
        if self.canvas.l_to_r:
            self.canvas.l_to_r = False
            self.toolbar.lr_button.setTitle_('L<-R')
            menu = NSApp.mainMenu().itemAtIndex_(2).submenu()
            menu.itemAtIndex_(0).setKeyEquivalent_(NSLeftArrowFunctionKey)
            menu.itemAtIndex_(1).setKeyEquivalent_(NSLeftArrowFunctionKey)
            menu.itemAtIndex_(2).setKeyEquivalent_(NSRightArrowFunctionKey)
            menu.itemAtIndex_(3).setKeyEquivalent_(NSRightArrowFunctionKey)
            menu.itemAtIndex_(5).setKeyEquivalent_(NSRightArrowFunctionKey)
            menu.itemAtIndex_(6).setKeyEquivalent_(NSLeftArrowFunctionKey)
        else:
            self.canvas.l_to_r = True
            self.toolbar.lr_button.setTitle_('L->R')
            menu = NSApp.mainMenu().itemAtIndex_(2).submenu()
            menu.itemAtIndex_(0).setKeyEquivalent_(NSRightArrowFunctionKey)
            menu.itemAtIndex_(1).setKeyEquivalent_(NSRightArrowFunctionKey)
            menu.itemAtIndex_(2).setKeyEquivalent_(NSLeftArrowFunctionKey)
            menu.itemAtIndex_(3).setKeyEquivalent_(NSLeftArrowFunctionKey)
            menu.itemAtIndex_(5).setKeyEquivalent_(NSLeftArrowFunctionKey)
            menu.itemAtIndex_(6).setKeyEquivalent_(NSRightArrowFunctionKey)
        if len(self.archive) != 0:
            self.canvas.display()

    def onThumbButtonClicked_(self, button):
        if len(self.archive) == 0:
            return
        collection = ComipoliCollection.newWithFrame_pSize_atOwner_(self.frame(), NSApp.thumbnail_height, self)
        collection.set_array(self.thumbnail_array)
        NSApp.runModalForWindow_(collection)
        self.makeKeyAndOrderFront_(self)

    def windowDidResize_(self, sender):
        self.canvas.setFrameSize_(self.contentView().frame().size)

    def windowWillEnterFullScreen_(self, sender):
        self.toolbar.setVisible_(False)
        self.is_fullscreen = True

    def windowDidExitFullScreen_(self, sender):
        self.toolbar.setVisible_(True)
        self.is_fullscreen = False

    def wantsPeriodicDraggingUpdates_(self, sender):
        return True

    def performDragOperation_(self, sender):
        pb = sender.draggingPasteboard()
        if pb.types().containsObject_(NSURLPboardType):
            url = NSURL.URLFromPasteboard_(pb)
            self.set_path(url.path())
        return True

    def draggingEntered_(self, sender):
        return NSDragOperationCopy

    def windowWillClose_(self, sender):
        defaults = NSUserDefaults.alloc().initWithSuiteName_('com.sasakima.comipoli')
        defaults.setFloat_forKey_(self.contentView().frame().size.width, 'width')
        defaults.setFloat_forKey_(self.contentView().frame().size.height - 22, 'height')
        defaults.setFloat_forKey_(self.frame().origin.x, 'x')
        defaults.setFloat_forKey_(self.frame().origin.y + 38, 'y')
        defaults.setBool_forKey_(self.canvas.l_to_r, 'LtoR')
        defaults.synchronize()
